
## **JavaScript Code Review 指南規範**

### **1. 語法與格式 (Syntax and Formatting)**

1.1 **縮排**

-   使用 2 或 4 個空格進行縮排，避免使用制表符 (Tab)。
-   確保縮排一致，所有代碼層級清晰易讀。

1.2 **行長度**

-   單行代碼限制在 80-120 個字符之內，超過需換行，並保持換行後的縮排一致。

1.3 **空白與空行**

-   關鍵字後需加一個空格，例如：`if (condition)` 而非 `if(condition)`。
-   使用空行分隔不同邏輯區塊，例如函式之間或程式碼段落之間。

1.4 **文件結尾**

-   文件以單個空行結束，避免多餘空行。

1.5 **標點符號**

-   始終使用分號 `;` 結束語句，除非使用 Prettier 等工具自動管理。
-   確保使用一致的引號（單引號 `'` 或雙引號 `"`），推薦單引號。

----------

### **2. 命名規範 (Naming Conventions)**

2.1 **變數與函式**

-   使用駝峰式命名 (camelCase)，如：`userName`、`fetchData`。
-   函式名稱應反映動作，例如：`getUserData()`、`updateSettings()`。

2.2 **常數**

-   使用全大寫與底線命名，例如：`MAX_LIMIT`、`API_KEY`。

2.3 **檔案與資料夾**

-   檔案名稱使用小寫加中劃線，例如：`user-profile.js`。
-   避免無意義命名，如 `data.js`，應具體描述文件用途。

----------

### **3. 邏輯與結構 (Logic and Structure)**

3.1 **程式結構**

-   模組化代碼，避免所有邏輯集中於一個文件。
-   單一函式應完成單一職責（單一職責原則）。

3.2 **條件語句**

-   避免嵌套過深的條件，將條件簡化或提前返回 (early return)。
-   使用三元運算符僅適用於簡單條件，複雜邏輯應使用 `if-else`。

3.3 **迴圈**

-   優先使用 `map`、`filter`、`reduce` 等高階函式代替傳統迴圈 (`for` / `while`)。
-   避免在迴圈中執行過多操作，確保效率。

----------

### **4. 可讀性與可維護性 (Readability and Maintainability)**

4.1 **註解**

-   必要時添加有意義的註解，但避免過多，代碼應自解 (self-explanatory)。
-   使用 JSDoc 為函式添加說明與參數類型註解：
    
    ```javascript
    /**
     * 計算兩數相加的結果
     * @param {number} a - 第一個數字
     * @param {number} b - 第二個數字
     * @returns {number} - 相加結果
     */
    function add(a, b) {
      return a + b;
    }
    ``` 
    

4.2 **代碼分段**

-   長函式需拆分為多個較小函式，確保每段程式碼專注於單一目的。

----------

### **5. 性能 (Performance)**

5.1 **資源管理**

-   避免不必要的 DOM 操作，優化選擇器與操作方式。
-   對於頻繁觸發的事件 (如 `scroll` 或 `resize`)，使用防抖 (debounce) 或節流 (throttle) 處理。

5.2 **資料處理**

-   避免使用同步方式處理 I/O 操作，使用 `async/await` 或 Promises。

----------

### **6. 安全性 (Security)**

6.1 **輸入驗證**

-   確保對所有用戶輸入進行驗證，防止 XSS、SQL 注入等攻擊。

6.2 **敏感資料**

-   避免在代碼中暴露敏感信息（如 API 金鑰、密碼）。
-   使用環境變數 (environment variables) 管理機密資訊。

----------

### **7. 測試 (Testing)**

7.1 **單元測試**

-   確保每個函式、模組有對應的單元測試，覆蓋所有關鍵邏輯。

7.2 **測試工具**

-   使用適當的測試框架（如 Jest、Mocha）進行測試。

7.3 **程式碼覆蓋率**

-   目標覆蓋率應達到至少 80%，尤其是核心模組。

----------

### **8. 依賴管理 (Dependency Management)**

8.1 **依賴版本**

-   鎖定依賴版本 (`package-lock.json` 或 `yarn.lock`)，避免版本不一致。

8.2 **依賴審查**

-   定期審查並更新依賴，移除未使用的依賴，確保使用最新且安全的版本。

----------

### **9. 自動化工具 (Automation Tools)**

9.1 **靜態檢查**

-   配置 ESLint 進行代碼靜態檢查，確保一致性與最佳實踐。

9.2 **格式化工具**

-   使用 Prettier 自動格式化代碼，確保格式統一。

9.3 **CI/CD**

-   配置持續整合/部署流程，在提交代碼後自動執行測試與部署。

----------

### **10. 最後檢查清單 (Final Checklist)**

-   語法是否正確，代碼是否格式化？
-   是否遵守命名規範？
-   邏輯是否清晰且易於理解？
-   是否考慮性能與安全性？
-   測試覆蓋率是否達標？
-   依賴是否更新至最新安全版本？
-   是否通過所有自動化檢查？